#!/usr/bin/env python
# -*- coding: utf-8 -*-

# License Agreement
# This code is licensed under the outer restricted Tiss license:
#
#  Copyright [2014]-[2019] Thales Services under the Thales Inner Source Software License
#  (Version 1.0, InnerPublic -OuterRestricted the "License");
#
#  You may not use this file except in compliance with the License.
#
#  The complete license agreement can be requested at contact@punchplatform.com.
#
#  Refer to the License for the specific language governing permissions and limitations
#  under the License.


formatters: dict[str, str] = {
    "RED": "\033[91m",
    "ERROR": "\033[91m",
    "GREEN": "\033[92m",
    "SUCCESS": "\033[92m",
    "YELLOW": "\033[33m",
    "WARNING": "\033[33m",
    "WHITE": "\033[37m",
    "BLUE": "\033[34m",
    "INFO": "\033[34m",
    "CYAN": "\033[36m",
    "MAGENTA": "\033[35m",
    "BLACK": "\033[30m",
    "DARKWHITE": "\033[37m",
    "DARKYELLOW": "\033[33m",
    "DARKGREEN": "\033[32m",
    "DARKBLUE": "\033[34m",
    "DARKCYAN": "\033[36m",
    "DARKRED": "\033[31m",
    "DARKMAGENTA": "\033[35m",
    "END": "\033[0m",
}


def color(apply_color: str, message: str) -> str:
    """apply color to a given message when output to terminal

    .. warning::

        Although this method is accessible as public, it is tagged as an internal one
        Changes might occur without notice

    :param apply_color: color code you want to apply to your message
    :param message: the actual string message to be colored
    :return value: inputted message will have added colored ascii

    Available colors are:
        * RED
        * GREEN
        * WHITE
        * YELLOW
        * BLUE
        * CYAN
        * MAGENTA
        * BLACK
        * DARKWHITE
        * DARKYELLOW
        * DARKGREEN
        * DARKBLUE
        * DARKCYAN
        * DARKRED
        * DARKMAGENTA
    """

    prefix_colored_string: str = "{" + apply_color + "}"
    prefix_colored_string = prefix_colored_string.format(**formatters)
    suffix_colored_string = "{END}"
    suffix_colored_string = suffix_colored_string.format(**formatters)
    colored_string = prefix_colored_string + message + suffix_colored_string
    return colored_string


def print_debug(message: str) -> None:
    """Add debug color to message and then prints it on stdout

    :color value: No color is applied

    :param message: Debug colored message
    :return value: Display textual information on terminal without returning anything
    """
    print(message)


def print_error(message: str) -> None:
    """Add error color to message and then prints it on stdout

    :color value: RED

    :param message: Debug colored message
    :return value: Display textual information on terminal without returning anything
    """
    print(color("RED", "ERROR: %s" % message))


def print_message(message: str) -> None:
    """Add color to message and then prints it on stdout

    :color value: DARKBLUE

    :param message: Debug colored message
    :return value: Display textual information on terminal without returning anything
    """
    print(color("DARKBLUE", message))


def print_success(message: str) -> None:
    """Add success color to message and then prints it on stdout

    :color value: GREEN

    :param message: Debug colored message
    :return value: Display textual information on terminal without returning anything
    """
    print(color("GREEN", message))


def print_warning(message: str) -> None:
    """Add warning color to message and then prints it on stdout

    :color value: DARKYELLOW

    :param message: Debug colored message
    :return value: Display textual information on terminal without returning anything
    """
    print(color("DARKYELLOW", "WARNING: %s" % message))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
The punch prometheus context makes it easier to correctly name the metrics,
in particular the usage of the namespace and subsystem, and the
right use of labels.

Using the plain prometheus client API, each metric user must
go through a child metric to publish values with the right labels.

Besides, the punch API uses a light facade API which makes it
future-proof to accommodate other metric back ends.
"""

__author__ = "RT"

import prometheus_client

import punch_api.metrics.counter
import punch_api.metrics.gauge
import punch_api.metrics.histogram


class MetricContext:
    counters: dict[str, prometheus_client.Counter]
    gauges: dict[str, prometheus_client.Gauge]
    histograms: dict[str, prometheus_client.Histogram]
    __labels: dict[str, str]
    __subsystem: str
    __namespace: str

    def __init__(
        self,
        namespace: str | None,
        subsystem: str | None,
        labels: dict[str, str] = None,
    ) -> None:
        self.__subsystem = "" if subsystem is None else subsystem
        self.__namespace = "" if namespace is None else namespace
        self.__labels = {} if labels is None else labels

    def get_new_subsystem_context(self, subsystem: str) -> "MetricContext":
        """
        Derive a context from a parent to add some specific tags for a subsystem.
        :param subsystem: the subsystem, typically netty, kafka, elasticsearch
        :return: a new context instance
        """
        return MetricContext(self.__namespace, subsystem, self.__labels)

    def add_tag(self, key: str, value: str) -> None:
        """Update the tag if it exists, otherwise add it"""
        self.__labels[key] = value

    def __get_labels(self) -> list[str]:
        return list(self.__labels.keys())

    def __get_label_values(self) -> list[str]:
        return list(self.__labels.values())

    def start(self, port: int) -> "MetricContext":
        if 0 < port < 65536:
            prometheus_client.start_http_server(port)
        else:
            raise ValueError(
                f"Bad port value: got {port}, expected value between 1 and 65535."
            )

        return self

    def get_counter(
        self, name: str, doc: str, unit: str
    ) -> punch_api.metrics.counter.Counter:
        return punch_api.metrics.counter.Counter(
            self.__get_create_registered_counter(name, doc, unit),
            self.__get_label_values(),
        )

    def get_histogram(
        self, name: str, doc: str, unit: str
    ) -> punch_api.metrics.histogram.Histogram:
        return punch_api.metrics.histogram.Histogram(
            self.__get_create_registered_histogram(name, doc, unit),
            self.__get_label_values(),
        )

    def get_gauge(
        self, name: str, doc: str, unit: str
    ) -> punch_api.metrics.gauge.Gauge:
        return punch_api.metrics.gauge.Gauge(
            self.__get_create_registered_gauge(name, doc, unit),
            self.__get_label_values(),
        )

    def __get_create_registered_counter(
        self, name: str, doc: str, unit: str
    ) -> prometheus_client.Counter:
        key: str = self.__get_metric_name(name, unit)
        if key not in self.counters:
            self.counters[key] = prometheus_client.Counter(
                name=name,
                subsystem=self.__subsystem,
                namespace=self.__namespace,
                unit=unit,
                labelnames=self.__get_labels(),
                documentation=doc,
            )
        return self.counters[key]

    def __get_create_registered_gauge(
        self, name: str, doc: str, unit: str
    ) -> prometheus_client.Gauge:
        key: str = self.__get_metric_name(name, unit)
        if key not in self.gauges:
            self.gauges[key] = prometheus_client.Gauge(
                name=name,
                subsystem=self.__subsystem,
                namespace=self.__namespace,
                unit=unit,
                labelnames=self.__get_labels(),
                documentation=doc,
            )
        return self.gauges[key]

    def __get_create_registered_histogram(
        self, name: str, doc: str, unit: str
    ) -> prometheus_client.Histogram:
        key: str = self.__get_metric_name(name, unit)
        if key not in self.histograms:
            self.histograms[key] = prometheus_client.Histogram(
                name=name,
                subsystem=self.__subsystem,
                namespace=self.__namespace,
                unit=unit,
                labelnames=self.__get_labels(),
                documentation=doc,
            )
        return self.histograms[key]

    def __get_metric_name(self, name: str, unit: str) -> str:
        sb: str = self.__namespace
        if len(self.__subsystem) > 0:
            sb += "_" + self.__subsystem
        sb += "_" + name
        if len(unit) > 0:
            sb += "_" + unit
        return sb



import prometheus_client


class Counter:
    child: prometheus_client.Counter

    def __init__(self, c: prometheus_client.Counter, labels: list[str]) -> None:
        self.child = c.labels(*labels)

    def inc(self) -> None:
        self.child.inc()
